<?php
define('SCAN_PING','-sn');
define('SCAN_SNMP','-sU -sS -p U:161,T:80,T:443,T:8080');

class Addresses{
	private $_addresses = array();

	public static function validate_IPv4_net($network_addr){
		$splitted = explode("/",$network_addr);

		if(count($splitted) != 2 ) return  false;

		$addr=$splitted[0];
		$cidr=$splitted[1];

		if($cidr<0 || $cidr > 32)	return false;
		return filter_var($addr,FILTER_VALIDATE_IP);
	}

	public function __construct($first_addr,$last_addr=null){
		if($last_addr==null) {
			if(is_string($first_addr))	$first_addr = explode(',',$first_addr);
			$this->addAddrArray($first_addr);
		}
		else{
			$this->addRange($first_addr, $last_addr);
		}
	}

	public function addAddr($addresse){
		$addresse = explode("-",$addresse);
		if(count($addresse) > 2) throw new Exception("Incorrect IP or network addr $addresse");
		if(count($addresse)==2) {
			$this->addRange($addresse[0],$addresse[1]);
			return;
		}
		$addresse = $addresse[0];

		if(isset($this->_addresses[$addresse]))
			throw new Exception("The addresses $addresse is already set");

			if(self::validate_IPv4_net($addresse)||filter_var($addresse,FILTER_VALIDATE_IP))
				$this->_addresses[$addresse]=$addresse;
				else
					throw new Exception("Incorrect IP or network addr $addresse");

	}

	public function addAddrArray(array $addresses){
		foreach($addresses as $addr){
			$this->addAddr($addr);
		}
	}

	public function addRange($first_addr,$last_addr){
		$first_int=ip2long($first_addr);
		$last_int=ip2long($last_addr);

		if(!$first_int) 			throw new Exception("Incorrect IP $first_addr");
		if(!$last_int)				throw new Exception("Incorrect IP $last_addr");
		if($first_int>$last_int)	throw new Exception("$first_addr must be before $last_addr");

		for($ip_num=$first_int;$ip_num<=$last_int;$ip_num++){
			$addr = long2ip($ip_num);
				
			if(isset($this->_addresses[$addr]))
				throw new Exception("The addresses $addr is already set");
				$this->_addresses[$addr]=$addr;
		}
	}

	public function getString(){
		return implode(" ",$this->_addresses);
	}
}

class ScanResult{
	private $_data;
	private $_command;

	private function parseParams($dom){
		$item = array();
		foreach($dom->attributes as $attribut){
			$item[$attribut->nodeName] = $attribut->nodeValue;
		}
		return $item;
	}

	private function parseNode($dom){
		$node = $this->parseParams($dom);
		foreach($dom->childNodes as $item){
			if( $item-> nodeType == XML_ELEMENT_NODE ){
				if(empty($node[$item->nodeName])) $node[$item->nodeName] = array();
				array_push($node[$item->nodeName],$this->parseNode($item));
			}
		}
		return $node;
	}

	public function __construct($data,$command){
		$this->_command = $command;

		$dom = new DOMDocument();
		$dom->loadXML($data);

		foreach($dom->childNodes as $item){
			if($item->nodeType ==  XML_ELEMENT_NODE && $item->nodeName == 'nmaprun' ){
				$this->_data = $this->parseNode($item);
			}
		}
	}

	public function getMetadata(){
		//$metadata['command']		= $this->_command;
		$metadata['started_date'] 	= $this->_data['start'];
		$metadata['stopped_date']   = $this->_data['runstats'][0]['finished'][0]['time'];
		$metadata['exit']           = $this->_data['runstats'][0]['finished'][0]['exit'];
		$metadata['host_up']       	= $this->_data['runstats'][0]['hosts'][0]['up'];
		$metadata['host_down']     	= $this->_data['runstats'][0]['hosts'][0]['down'];
		$metadata['host_total']    	= $this->_data['runstats'][0]['hosts'][0]['total'];
		return $metadata;
	}

	private function getAddrType($host,$type){
		foreach($host['address'] as $addr){
			if($addr['addrtype'] == $type ) return $addr['addr'];
		}
		return '';
	}


	private function getPortsList($host){
		$ret = array();
		if(empty($host['ports'])) return $ret;
		foreach($host['ports'] as $ports){
			foreach($ports['port'] as $port){
				$row['protocol']= $port['protocol'];
				$row['portid']	= $port['portid'];
				$row['status'] 	= $port['state'][0]['state'];
				$row['reason'] 	= $port['state'][0]['reason'];
				$ret[] = $row;
			}
		}
		return $ret;
	}

	public function getHosts(){
		$ret = array();
		foreach($this->_data['host'] as $host){
			if($host['status'][0]['state'] == 'up' ){
				$row['address'] = $this->getAddrType($host,'ipv4');
				$row['mac']        = $this->getAddrType($host,'mac');

				if(isset($host['hostnames'][0]['hostname'][0]['name']))
					$row['hostname'] = $host['hostnames'][0]['hostname'][0]['name'];
					else $row['hostname']='';
					$row['ports'] = $this->getPortsList($host);
					array_push($ret,$row);
			}
		}
		return $ret;
	}

	public function getData(){
		$data = array();
		$data["metadata"] 	= $this->getMetaData();
		$data["hosts"]		= $this->getHosts();
		return $data;
	}
}

class Nmap{
	private $_tmp_dir="/tmp";
	private $_command="/usr/bin/nmap";
	private $_scan_type=SCAN_PING;

	public function __construct($scan_type=null,$command=null){
		if($scan_type!=null) $this->_scan_type=$scan_type;
		if($command!=null)	 $this->_command=$command;
	}

	public function start_scan($addresses){
		$tmp_file = tempnam($this->_tmp_dir,"NmapLib");

		if(!is_a($addresses,"Addresses"))	$addresses=new Addresses($addresses);

		$started_time = microtime(true);

		$full_command=sprintf("%s %s -oX %s %s 2>&1 >/dev/null",$this->_command,$this->_scan_type,$tmp_file,$addresses->getString());
		system($full_command);
		$data=file_get_contents($tmp_file);

		$end_time = microtime(true);

		unlink($tmp_file);

		return new ScanResult($data,$full_command);
	}
}

class ScanSnmp{
	const V1 = 1;
	const V2C = 2;

	private $_oid;
	private $_timeouts;

	private $_community_found;
	private $_oid_value;

	private $_nmap;

	public function __construct($oid='1.3.6.1.2.1.1.2',$first_timeout = '1000000' , $second_timeout = '2000000'){
		$this->_oid = $oid;
		$this->_timeouts = array($first_timeout,$second_timeout);
		$this->_community_found = array();
		$this->_oid_value = array();
		$this->_nmap = new Nmap(SCAN_SNMP);
	}

	private function detect_community($host,$communities,$version,$timeout){
		if(is_string($communities)) $communities = explode(',', $communities);

		foreach($communities as $community){
			if( $version == self::V1 )  $ret = @array_shift(snmprealwalk($host,$community,$this->_oid,$timeout,0));
			else 						$ret = @array_shift(snmp2_real_walk($host,$community,$this->_oid,$timeout,0));
				
			if($ret != false){
				$this->_community_found[$host] = $community;
				$this->_oid_value[$host] = $ret;
			}
		}
	}

	private function isCommunityFound($host){
		return (isset($this->_community_found[$host]));
	}

	private function snmp_details($host,$com,$ver,$timeout){
		$oids = array(
				'1.3.6.1.2.1.1.1'=>'sysDescr',
				'1.3.6.1.2.1.1.2'=>'sysObjectID',
				'1.3.6.1.2.1.1.3'=>'sysUpTime',
				'1.3.6.1.2.1.1.4'=>'sysContact',
				'1.3.6.1.2.1.1.5'=>'sysName',
				'1.3.6.1.2.1.1.6'=>'sysLocation',
				'1.3.6.1.2.1.1.7'=>'sysServices'
		);

		foreach($oids as $oid => $desc){
			if($oid == $this->_oid && isset($this->_oid_value[$host])){ //Pas besoin de refaire la requête si on a déjà l'info
				$ret = $this->_oid_value[$host];
			}
			else{
				if( $ver == self::V1 )  	$ret = @array_shift(snmprealwalk($host,$com,$oid,$timeout,0));
				else 						$ret = @array_shift(snmp2_real_walk($host,$com,$oid,$timeout,0));
			}
				
			$details[$desc] = $ret;
		}
		return $details;
	}

	public function scan_snmp($host,$communities){
		if(is_string($communities)) $communities = explode(',', $communities);

		$first_timeout	= $this->_timeouts[0];
		$second_timeout = $this->_timeouts[1];

		$detected['status'] = 'no';

		$ver = null;
		$this->detect_community($host,$communities,self::V2C,$first_timeout);
		if( !$this->isCommunityFound($host) ) $this->detect_community($host,$communities,self::V2C,$second_timeout);

		if( $this->isCommunityFound($host) ) $ver = 2;
		else {
			$this->detect_community($host,$communities,self::V1,$first_timeout);
			if( !$this->isCommunityFound($host) ) $this->detect_community($host,$communities,self::V1,$second_timeout);
			if( $this->isCommunityFound($host) ) $ver = 1;
		}

		if( $ver != null ) {
			$detected['status'] = 'yes';
			$detected['version'] = 2;
			$detected['community'] = $this->_community_found[$host];

			$detected['details'] = $this->snmp_details($host,$this->_community_found[$host],$ver,$second_timeout);
		}
		return $detected;
	}

	public function full_scan($addresses,$communities){
		$data = $this->_nmap->start_scan($addresses)->getData();
		for($i=0;$i<count($data['hosts']);$i++){
			$data['hosts'][$i]['snmp']['status']='no';
			foreach($data['hosts'][$i]['ports'] as $port){
				if($port['portid'] == 161 && $port['status'] != 'closed' ){
					$data['hosts'][$i]['snmp'] = $this->scan_snmp($data['hosts'][$i]['address'],$communities);
				}
			}
		}
		return $data;
	}
}

define('DATA_DIR',"/tmp/esia_scan");

class task{
	public static $PENDING="pending";
	public static $PROCESS="processing";
	public static $TERMINATE="terminated";
	public static $ERROR="error";

	private $_path;
	private $_addresses;

	public static function getPendingTasks(){
		$tasks = array();

		$files = @scandir(DATA_DIR,SCANDIR_SORT_ASCENDING);
		if(!$files) throw new Exception('Unable to read '.DATA_DIR.' directory');
		foreach($files as $file){
			$full_path = DATA_DIR."/$file";
			if(!is_file($full_path)) continue; //C'est un dossier
			if(!is_writable($full_path)) {
				echo "Ignore unwritable file $full_path";
				continue;
			}
				
			$raw = file_get_contents($full_path);
			$parsed = json_decode($raw,true);
			if(!isset($parsed["status"])){
				echo "Ignore incorrect file format $full_path";
				continue;
			}
			if($parsed["status"] == self::$PENDING) $tasks[] = new task($full_path);
		}
		return $tasks;
	}

	public function __construct($path){
		$this->_path = $path;
	}

	public function getName(){
		return basename($this->_path);
	}

	public function prepare_process_action(){
		$raw = file_get_contents($this->_path);
		$parsed = json_decode($raw,true);

		if(!isset($parsed["status"]))			throw new Exception('Incorrect format for file '.$this->_path);
		if($parsed["status"] != self::$PENDING)	throw new Exception("This action must be execute on a PENDING task");
		if(!isset($parsed["addresses"]))		throw new Exception('Incorrect format for file '.$this->_path);

		$parsed["status"] = self::$PROCESS;
		$parsed["since_status"]	  = time();
		file_put_contents($this->_path,json_encode($parsed,JSON_PRETTY_PRINT)."\n");

		$ret["addresses"] = $parsed["addresses"];

		if(empty($parsed["type"]) ) $ret["type"] = "ICMP";
		else $ret["type"] =  strtoupper($parsed["type"]);

		if(isset($parsed["options"])) $ret["options"] = $parsed["options"];

		return $ret;
	}

	public function end_process_action($result){
		$raw = file_get_contents($this->_path);
		$parsed = json_decode($raw,true);

		if(!isset($parsed["status"]))			throw new Exception('Incorrect format for file '.$this->_path);
		if($parsed["status"] != self::$PROCESS)	throw new Exception("This action must be execute on a PROCESSING task");

		$parsed["status"] = self::$TERMINATE;

		unset($parsed["since_status"]);

		$parsed = array_merge($parsed,$result);
		file_put_contents($this->_path,json_encode($parsed,JSON_PRETTY_PRINT)."\n");
	}

	public function failure_process_action($msg){
		$raw = file_get_contents($this->_path);
		$parsed = json_decode($raw,true);

		if(isset($parsed["status"]))	$parsed["before_failure_status"] = $parsed["status"];
		$parsed["status"] = self::$ERROR;
		$parsed["error_msg"] = $msg;

		file_put_contents($this->_path,json_encode($parsed,JSON_PRETTY_PRINT)."\n");
	}
}

try{
	$nmap = new Nmap();
	$snmp = new ScanSnmp();
	$tasks = task::getPendingTasks();
	/*
	 if tasks->getType == PING
	 startsca Namp
	 elsif(task->getType SNMP
	 	
	 elsif($task->getType == WMI

	 esif ($task->getType == ESX

	 $result
	 */
	if(count($tasks)==0) echo "No task found ".count($tasks)."\n";

	foreach($tasks as $task){
		try{
			echo 'Run task '.$task->getName()."\n";

			$params = $task->prepare_process_action();
			$addresses = $params["addresses"];
			if($params['type']=='SNMP') $data = $snmp->full_scan($addresses,$params['options']);
			else {
				$result = $nmap->start_scan($addresses);
				$data = $result->getData();
			}
			$task->end_process_action($data);
				
			echo 'Task : '.$task->getName()." terminated \n";
		}
		catch(Exception $ex){
			$task->failure_process_action($ex->getMessage());
			echo "EXCEPTION = ".$ex->getLine().":".$ex->getMessage()."\n";
		}
	}
}
catch(Exception $ex){
	echo "EXCEPTION = ".$ex->getLine().":".$ex->getMessage()."\n";
}
?>
